/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

namespace fe
{

void RasterOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<String>("ValueAttr")="value";
	catalog<String>("ValueAttr","label")="Value Attribute";
	catalog<String>("ValueAttr","hint")=
			"Name of point attribute to place results.";

	catalog<String>("savefile");
	catalog<String>("savefile","label")="Save File";

	catalog< sp<Component> >("Input Surface");
}

void RasterOp::handle(Record& a_rSignal)
{
	const String valueAttr=catalog<String>("ValueAttr");
	const String savefile=catalog<String>("savefile");

	if(savefile.empty())
	{
		catalog<String>("warning")+="no filename for saving image;";
		return;
	}

	sp<SurfaceAccessibleI> spOutputAccessible;
	if(!accessOutput(spOutputAccessible,a_rSignal)) return;

	sp<SurfaceAccessorI> spOutputColor;
	if(!access(spOutputColor,spOutputAccessible,
			e_point,"Cd")) return;

	sp<SurfaceAccessibleI> spInputAccessible;
	if(!access(spInputAccessible,"Input Surface")) return;

	sp<SurfaceAccessorI> spInputWidth;
	if(!access(spInputWidth,spInputAccessible,
			e_detail,"raster_width")) return;

	sp<SurfaceAccessorI> spInputHeight;
	if(!access(spInputHeight,spInputAccessible,
			e_detail,"raster_height")) return;

	sp<SurfaceAccessorI> spInputMin;
	if(!access(spInputMin,spInputAccessible,
			e_detail,"raster_min")) return;

	sp<SurfaceAccessorI> spInputMax;
	if(!access(spInputMax,spInputAccessible,
			e_detail,"raster_max")) return;

	sp<SurfaceAccessorI> spInputValue;
	if(!access(spInputValue,spInputAccessible,e_point,valueAttr)) return;

	const I32 width=spInputWidth->integer(0);
	const I32 height=spInputHeight->integer(0);
	const I32 min=spInputMin->integer(0);
	const I32 max=spInputMax->integer(0);

	sp<ImageI> spImage=registry()->create("*.ImageIL");
	if(spImage.isNull())
	{
		catalog<String>("error")+="could not imitialize image;";
		return;
	}

	U8 data[height][width][3];

	const U32 pointCount=spInputValue->count();
	for(U32 pointIndex=0;pointIndex<pointCount;pointIndex++)
	{
		const U32 uIndex=pointIndex%width;
		const U32 vIndex=pointIndex/width;

		if(uIndex>=width || vIndex>=height)
		{
			feLog("%d u %d/%d v %d/%d\n",
					pointIndex,uIndex,width,vIndex,height);
			continue;
		}

		U8* pPixel=data[vIndex][uIndex];

		//* TODO verify correct uv
//		const Real uCoord=uIndex/(width-1.0);
//		const Real vCoord=vIndex/(height-1.0);

		const Real value=spInputValue->real(pointIndex);

		I32 byte=255*(value-min)/(max-min);
		if(byte<0)
		{
			byte=0;
		}
		if(byte>255)
		{
			byte=255;
		}

		pPixel[0]=byte;
		pPixel[1]=value<0.0? 0: byte;
		pPixel[2]=byte>250? byte: 0;

		const Color color=(1.0/255.0)*Color(pPixel[0],pPixel[1],pPixel[2]);
		spOutputColor->set(pointIndex,color);

//		feLog("%d u %d/%d v %d/%d bytes %d %d %d\n",
//				pointIndex,uIndex,width,vIndex,height,
//				pPixel[0],pPixel[1],pPixel[2]);
	}

	spImage->createSelect();
	spImage->setFormat(ImageI::e_rgb);
	spImage->resize(width,height,1);
	spImage->replaceRegion(0,0,0,width,height,1,data);
	const BWORD success=spImage->save(savefile);
}

} // namespace
